@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2017 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2018 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -27,6 +27,7 @@ import getpass
 import fnmatch
 import traceback
 import datetime
+import enum
 
 import pkg_resources
 from PyQt5.QtCore import pyqtSlot, Qt, QSize
@@ -35,14 +36,14 @@ from PyQt5.QtWidgets import (QDialog, QLabel, QTextEdit, QPushButton,
                              QDialogButtonBox, QApplication, QMessageBox)
 
 import qutebrowser
-from qutebrowser.utils import version, log, utils, objreg, usertypes
+from qutebrowser.utils import version, log, utils, objreg
 from qutebrowser.misc import (miscwidgets, autoupdate, msgbox, httpclient,
                               pastebin)
 from qutebrowser.config import config, configfiles
 
 
-Result = usertypes.enum('Result', ['restore', 'no_restore'], is_int=True,
-                        start=QDialog.Accepted + 1)
+Result = enum.IntEnum('Result', ['restore', 'no_restore'],
+                      start=QDialog.Accepted + 1)
 
 
 def parse_fatal_stacktrace(text):
@@ -56,17 +57,22 @@ def parse_fatal_stacktrace(text):
         element being the first stacktrace frame.
     """
     lines = [
-        r'Fatal Python error: (.*)',
+        r'(?P<type>Fatal Python error|Windows fatal exception): (?P<msg>.*)',
         r' *',
         r'(Current )?[Tt]hread [^ ]* \(most recent call first\): *',
-        r'  File ".*", line \d+ in (.*)',
+        r'  File ".*", line \d+ in (?P<func>.*)',
     ]
-    m = re.match('\n'.join(lines), text)
+    m = re.search('\n'.join(lines), text)
     if m is None:
         # We got some invalid text.
         return ('', '')
     else:
-        return (m.group(1), m.group(3))
+        msg = m.group('msg')
+        typ = m.group('type')
+        func = m.group('func')
+        if typ == 'Windows fatal exception':
+            msg = 'Windows ' + msg
+        return msg, func
 
 
 def _get_environment_vars():
@@ -342,7 +348,6 @@ class _CrashDialog(QDialog):
                          "but you're currently running v{} - please "
                          "update!".format(newest, qutebrowser.__version__))
         text = '<br/><br/>'.join(lines)
-        self.finish()
         msgbox.information(self, "Report successfully sent!", text,
                            on_finished=self.finish, plain_text=False)
 
@@ -359,7 +364,6 @@ class _CrashDialog(QDialog):
                      "<a href=https://www.qutebrowser.org/>qutebrowser.org</a> "
                      "by yourself.".format(msg))
         text = '<br/><br/>'.join(lines)
-        self.finish()
         msgbox.information(self, "Report successfully sent!", text,
                            on_finished=self.finish, plain_text=False)
 
@@ -473,7 +477,8 @@ class FatalCrashDialog(_CrashDialog):
         self._type, self._func = parse_fatal_stacktrace(self._log)
 
     def _get_error_type(self):
-        if self._type == 'Segmentation fault':
+        if self._type in ['Segmentation fault',
+                          'Windows access violation']:
             return 'segv'
         else:
             return self._type
@@ -521,7 +526,7 @@ class FatalCrashDialog(_CrashDialog):
         """Prevent empty reports."""
         if (not self._info.toPlainText().strip() and
                 not self._contact.toPlainText().strip() and
-                self._type == 'Segmentation fault' and
+                self._get_error_type() == 'segv' and
                 self._func == 'qt_mainloop'):
             msgbox.msgbox(parent=self, title='Empty crash info',
                           text="Empty reports for fatal crashes are useless "
