@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2017 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2018 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -20,23 +20,22 @@
 """Other utilities which don't fit anywhere else."""
 
 import os
+import os.path
 import io
 import re
 import sys
 import enum
 import json
-import os.path
-import collections
 import datetime
 import traceback
 import functools
 import contextlib
 import socket
 import shlex
+import glob
 
-import attr
-from PyQt5.QtCore import Qt, QUrl
-from PyQt5.QtGui import QKeySequence, QColor, QClipboard, QDesktopServices
+from PyQt5.QtCore import QUrl
+from PyQt5.QtGui import QColor, QClipboard, QDesktopServices
 from PyQt5.QtWidgets import QApplication
 import pkg_resources
 import yaml
@@ -48,11 +47,12 @@ except ImportError:  # pragma: no cover
     YAML_C_EXT = False
 
 import qutebrowser
-from qutebrowser.utils import qtutils, log, debug
+from qutebrowser.utils import qtutils, log
 
 
 fake_clipboard = None
 log_clipboard = False
+_resource_cache = {}
 
 is_mac = sys.platform.startswith('darwin')
 is_linux = sys.platform.startswith('linux')
@@ -60,6 +60,11 @@ is_windows = sys.platform.startswith('win')
 is_posix = os.name == 'posix'
 
 
+class Unreachable(Exception):
+
+    """Raised when there was unreachable code."""
+
+
 class ClipboardError(Exception):
 
     """Raised if the clipboard contents are unavailable for some reason."""
@@ -137,6 +142,15 @@ def compact_text(text, elidelength=None):
     return out
 
 
+def preload_resources():
+    """Load resource files into the cache."""
+    for subdir, pattern in [('html', '*.html'), ('javascript', '*.js')]:
+        path = resource_filename(subdir)
+        for full_path in glob.glob(os.path.join(path, pattern)):
+            sub_path = '/'.join([subdir, os.path.basename(full_path)])
+            _resource_cache[sub_path] = read_file(sub_path)
+
+
 def read_file(filename, binary=False):
     """Get the contents of a file contained with qutebrowser.
 
@@ -148,6 +162,9 @@ def read_file(filename, binary=False):
     Return:
         The file contents as string.
     """
+    if not binary and filename in _resource_cache:
+        return _resource_cache[filename]
+
     if hasattr(sys, 'frozen'):
         # PyInstaller doesn't support pkg_resources :(
         # https://github.com/pyinstaller/pyinstaller/wiki/FAQ#misc
@@ -280,263 +297,6 @@ def format_size(size, base=1024, suffix=''):
     return '{:.02f}{}{}'.format(size, prefixes[-1], suffix)
 
 
-def key_to_string(key):
-    """Convert a Qt::Key member to a meaningful name.
-
-    Args:
-        key: A Qt::Key member.
-
-    Return:
-        A name of the key as a string.
-    """
-    special_names_str = {
-        # Some keys handled in a weird way by QKeySequence::toString.
-        # See https://bugreports.qt.io/browse/QTBUG-40030
-        # Most are unlikely to be ever needed, but you never know ;)
-        # For dead/combining keys, we return the corresponding non-combining
-        # key, as that's easier to add to the config.
-        'Key_Blue': 'Blue',
-        'Key_Calendar': 'Calendar',
-        'Key_ChannelDown': 'Channel Down',
-        'Key_ChannelUp': 'Channel Up',
-        'Key_ContrastAdjust': 'Contrast Adjust',
-        'Key_Dead_Abovedot': '˙',
-        'Key_Dead_Abovering': '˚',
-        'Key_Dead_Acute': '´',
-        'Key_Dead_Belowdot': 'Belowdot',
-        'Key_Dead_Breve': '˘',
-        'Key_Dead_Caron': 'ˇ',
-        'Key_Dead_Cedilla': '¸',
-        'Key_Dead_Circumflex': '^',
-        'Key_Dead_Diaeresis': '¨',
-        'Key_Dead_Doubleacute': '˝',
-        'Key_Dead_Grave': '`',
-        'Key_Dead_Hook': 'Hook',
-        'Key_Dead_Horn': 'Horn',
-        'Key_Dead_Iota': 'Iota',
-        'Key_Dead_Macron': '¯',
-        'Key_Dead_Ogonek': '˛',
-        'Key_Dead_Semivoiced_Sound': 'Semivoiced Sound',
-        'Key_Dead_Tilde': '~',
-        'Key_Dead_Voiced_Sound': 'Voiced Sound',
-        'Key_Exit': 'Exit',
-        'Key_Green': 'Green',
-        'Key_Guide': 'Guide',
-        'Key_Info': 'Info',
-        'Key_LaunchG': 'LaunchG',
-        'Key_LaunchH': 'LaunchH',
-        'Key_MediaLast': 'MediaLast',
-        'Key_Memo': 'Memo',
-        'Key_MicMute': 'Mic Mute',
-        'Key_Mode_switch': 'Mode switch',
-        'Key_Multi_key': 'Multi key',
-        'Key_PowerDown': 'Power Down',
-        'Key_Red': 'Red',
-        'Key_Settings': 'Settings',
-        'Key_SingleCandidate': 'Single Candidate',
-        'Key_ToDoList': 'Todo List',
-        'Key_TouchpadOff': 'Touchpad Off',
-        'Key_TouchpadOn': 'Touchpad On',
-        'Key_TouchpadToggle': 'Touchpad toggle',
-        'Key_Yellow': 'Yellow',
-        'Key_Alt': 'Alt',
-        'Key_AltGr': 'AltGr',
-        'Key_Control': 'Control',
-        'Key_Direction_L': 'Direction L',
-        'Key_Direction_R': 'Direction R',
-        'Key_Hyper_L': 'Hyper L',
-        'Key_Hyper_R': 'Hyper R',
-        'Key_Meta': 'Meta',
-        'Key_Shift': 'Shift',
-        'Key_Super_L': 'Super L',
-        'Key_Super_R': 'Super R',
-        'Key_unknown': 'Unknown',
-    }
-    # We now build our real special_names dict from the string mapping above.
-    # The reason we don't do this directly is that certain Qt versions don't
-    # have all the keys, so we want to ignore AttributeErrors.
-    special_names = {}
-    for k, v in special_names_str.items():
-        try:
-            special_names[getattr(Qt, k)] = v
-        except AttributeError:
-            pass
-    # Now we check if the key is any special one - if not, we use
-    # QKeySequence::toString.
-    try:
-        return special_names[key]
-    except KeyError:
-        name = QKeySequence(key).toString()
-        morphings = {
-            'Backtab': 'Tab',
-            'Esc': 'Escape',
-        }
-        if name in morphings:
-            return morphings[name]
-        else:
-            return name
-
-
-def keyevent_to_string(e):
-    """Convert a QKeyEvent to a meaningful name.
-
-    Args:
-        e: A QKeyEvent.
-
-    Return:
-        A name of the key (combination) as a string or
-        None if only modifiers are pressed..
-    """
-    if is_mac:
-        # Qt swaps Ctrl/Meta on macOS, so we switch it back here so the user
-        # can use it in the config as expected. See:
-        # https://github.com/qutebrowser/qutebrowser/issues/110
-        # http://doc.qt.io/qt-5.4/osx-issues.html#special-keys
-        modmask2str = collections.OrderedDict([
-            (Qt.MetaModifier, 'Ctrl'),
-            (Qt.AltModifier, 'Alt'),
-            (Qt.ControlModifier, 'Meta'),
-            (Qt.ShiftModifier, 'Shift'),
-        ])
-    else:
-        modmask2str = collections.OrderedDict([
-            (Qt.ControlModifier, 'Ctrl'),
-            (Qt.AltModifier, 'Alt'),
-            (Qt.MetaModifier, 'Meta'),
-            (Qt.ShiftModifier, 'Shift'),
-        ])
-    modifiers = (Qt.Key_Control, Qt.Key_Alt, Qt.Key_Shift, Qt.Key_Meta,
-                 Qt.Key_AltGr, Qt.Key_Super_L, Qt.Key_Super_R, Qt.Key_Hyper_L,
-                 Qt.Key_Hyper_R, Qt.Key_Direction_L, Qt.Key_Direction_R)
-    if e.key() in modifiers:
-        # Only modifier pressed
-        return None
-    mod = e.modifiers()
-    parts = []
-    for (mask, s) in modmask2str.items():
-        if mod & mask and s not in parts:
-            parts.append(s)
-    parts.append(key_to_string(e.key()))
-    return normalize_keystr('+'.join(parts))
-
-
-@attr.s(repr=False)
-class KeyInfo:
-
-    """Stores information about a key, like used in a QKeyEvent.
-
-    Attributes:
-        key: Qt::Key
-        modifiers: Qt::KeyboardModifiers
-        text: str
-    """
-
-    key = attr.ib()
-    modifiers = attr.ib()
-    text = attr.ib()
-
-    def __repr__(self):
-        if self.modifiers is None:
-            modifiers = None
-        else:
-            #modifiers = qflags_key(Qt, self.modifiers)
-            modifiers = hex(int(self.modifiers))
-        return get_repr(self, constructor=True,
-                        key=debug.qenum_key(Qt, self.key),
-                        modifiers=modifiers, text=self.text)
-
-
-class KeyParseError(Exception):
-
-    """Raised by _parse_single_key/parse_keystring on parse errors."""
-
-    def __init__(self, keystr, error):
-        super().__init__("Could not parse {!r}: {}".format(keystr, error))
-
-
-def is_special_key(keystr):
-    """True if keystr is a 'special' keystring (e.g. <ctrl-x> or <space>)."""
-    return keystr.startswith('<') and keystr.endswith('>')
-
-
-def _parse_single_key(keystr):
-    """Convert a single key string to a (Qt.Key, Qt.Modifiers, text) tuple."""
-    if is_special_key(keystr):
-        # Special key
-        keystr = keystr[1:-1]
-    elif len(keystr) == 1:
-        # vim-like key
-        pass
-    else:
-        raise KeyParseError(keystr, "Expecting either a single key or a "
-                            "<Ctrl-x> like keybinding.")
-
-    seq = QKeySequence(normalize_keystr(keystr), QKeySequence.PortableText)
-    if len(seq) != 1:
-        raise KeyParseError(keystr, "Got {} keys instead of 1.".format(
-            len(seq)))
-    result = seq[0]
-
-    if result == Qt.Key_unknown:
-        raise KeyParseError(keystr, "Got unknown key.")
-
-    modifier_mask = int(Qt.ShiftModifier | Qt.ControlModifier |
-                        Qt.AltModifier | Qt.MetaModifier | Qt.KeypadModifier |
-                        Qt.GroupSwitchModifier)
-    assert Qt.Key_unknown & ~modifier_mask == Qt.Key_unknown
-
-    modifiers = result & modifier_mask
-    key = result & ~modifier_mask
-
-    if len(keystr) == 1 and keystr.isupper():
-        modifiers |= Qt.ShiftModifier
-
-    assert key != 0, key
-    key = Qt.Key(key)
-    modifiers = Qt.KeyboardModifiers(modifiers)
-
-    # Let's hope this is accurate...
-    if len(keystr) == 1 and not modifiers:
-        text = keystr
-    elif len(keystr) == 1 and modifiers == Qt.ShiftModifier:
-        text = keystr.upper()
-    else:
-        text = ''
-
-    return KeyInfo(key, modifiers, text)
-
-
-def parse_keystring(keystr):
-    """Parse a keystring like <Ctrl-x> or xyz and return a KeyInfo list."""
-    if is_special_key(keystr):
-        return [_parse_single_key(keystr)]
-    else:
-        return [_parse_single_key(char) for char in keystr]
-
-
-def normalize_keystr(keystr):
-    """Normalize a keystring like Ctrl-Q to a keystring like Ctrl+Q.
-
-    Args:
-        keystr: The key combination as a string.
-
-    Return:
-        The normalized keystring.
-    """
-    keystr = keystr.lower()
-    replacements = (
-        ('control', 'ctrl'),
-        ('windows', 'meta'),
-        ('mod1', 'alt'),
-        ('mod4', 'meta'),
-    )
-    for (orig, repl) in replacements:
-        keystr = keystr.replace(orig, repl)
-    for mod in ['ctrl', 'meta', 'alt', 'shift']:
-        keystr = keystr.replace(mod + '-', mod + '+')
-    return keystr
-
-
 class FakeIOStream(io.TextIOBase):
 
     """A fake file-like stream which calls a function for write-calls."""
@@ -584,7 +344,7 @@ def disabled_excepthook():
             sys.excepthook = old_excepthook
 
 
-class prevent_exceptions:  # pylint: disable=invalid-name
+class prevent_exceptions:  # noqa: N801,N806 pylint: disable=invalid-name
 
     """Decorator to ignore and log exceptions.
 
@@ -631,6 +391,7 @@ class prevent_exceptions:  # pylint: disable=invalid-name
 
         @functools.wraps(func)
         def wrapper(*args, **kwargs):
+            """Call the original function."""
             try:
                 return func(*args, **kwargs)
             except BaseException:
@@ -869,7 +630,7 @@ def expand_windows_drive(path):
     # E:\. The correct way to specifify drive E: is "E:\", but most users
     # probably don't use the "multiple working directories" feature and expect
     # "E:" and "E:\" to be equal.
-    if re.match(r'[A-Z]:$', path, re.IGNORECASE):
+    if re.fullmatch(r'[A-Z]:', path, re.IGNORECASE):
         return path + "\\"
     else:
         return path
@@ -882,7 +643,7 @@ def yaml_load(f):
     end = datetime.datetime.now()
 
     delta = (end - start).total_seconds()
-    deadline = 3 if 'CI' in os.environ else 2
+    deadline = 5 if 'CI' in os.environ else 2
     if delta > deadline:  # pragma: no cover
         log.misc.warning(
             "YAML load took unusually long, please report this at "
@@ -909,3 +670,14 @@ def yaml_dump(data, f=None):
         return None
     else:
         return yaml_data.decode('utf-8')
+
+
+def chunk(elems, n):
+    """Yield successive n-sized chunks from elems.
+
+    If elems % n != 0, the last chunk will be smaller.
+    """
+    if n < 1:
+        raise ValueError("n needs to be at least 1!")
+    for i in range(0, len(elems), n):
+        yield elems[i:i + n]
